"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const __1 = require("../..");
const List = (props) => {
    const { indexList, dataList, renderItem, activeStickyIndex, onKeyIndexChange } = props;
    const [stickyIndex, setStickyIndex] = react_1.useState(-1);
    const $scroll = react_1.useRef(null);
    // 记录每个标头的index位置
    const keyIndexMap = react_1.useRef([]);
    // 记录每一项的高度
    const layoutMap = react_1.useRef([]);
    // 延迟渲染的定时器
    const timer = react_1.useRef(null);
    // 是否是外部滑动导致的滚动
    const isTouchingByAnchor = react_1.useRef(false);
    // 清除定时器
    react_1.useEffect(() => {
        return () => {
            timer.current && clearTimeout(timer.current);
        };
    }, []);
    // 外部标头变化导致滚动
    react_1.useEffect(() => {
        if (activeStickyIndex >= 0) {
            isTouchingByAnchor.current = true;
            setActiveGroupByIndex(activeStickyIndex);
        }
    }, [activeStickyIndex]);
    const handleDataList = (data) => {
        let res = [];
        let index = 0;
        data.forEach((item, idx) => {
            // 要注意不要赋值，否则会导致引用错误
            keyIndexMap.current.push({ index, offset: 0 });
            const name = indexList[idx];
            res.push({ type: 'anchor', name });
            item.forEach(i => {
                res.push({ type: 'item', data: i });
                index++;
            });
            index++;
        });
        return res;
    };
    const setActiveGroupByIndex = (index) => {
        const currentKeyMap = keyIndexMap.current[index];
        if (currentKeyMap && currentKeyMap.index !== undefined) {
            $scroll?.current?.scrollTo({
                y: currentKeyMap.offset,
                animated: false,
            });
            // 延迟设置stickyHeader，减少渲染，防止卡顿
            if (timer.current) {
                clearTimeout(timer.current);
            }
            timer.current = setTimeout(() => {
                isTouchingByAnchor.current = false;
                setStickyIndex(currentKeyMap.index);
            }, 300);
        }
    };
    const onItemLayout = (event, index, isLast) => {
        layoutMap.current[index] = { height: event.nativeEvent.layout.height };
        if (isLast) {
            const indexMap = keyIndexMap.current;
            const layout = layoutMap.current;
            indexMap.forEach((item, idx) => {
                const prev = idx === 0 ? undefined : indexMap[idx - 1];
                const start = prev ? prev.index : 0;
                let offset = prev ? prev.offset : 0;
                for (let i = start; i < item.index; i++) {
                    const each = layout[i];
                    offset += each.height;
                }
                keyIndexMap.current[idx] = {
                    ...item,
                    offset,
                };
            });
        }
    };
    const onScroll = (event) => {
        if (isTouchingByAnchor.current) {
            return false;
        }
        const { y } = event.nativeEvent.contentOffset;
        const indexMap = keyIndexMap.current;
        let stickyIndex = -1;
        let keyIndex = -1;
        for (let i = 0; i < indexMap.length; i++) {
            const item = indexMap[i];
            const { offset, index } = item;
            if (offset <= y) {
                stickyIndex = index;
                keyIndex = i;
            }
            else {
                break;
            }
        }
        if (stickyIndex !== -1) {
            setStickyIndex(stickyIndex);
            onKeyIndexChange(keyIndex);
        }
    };
    const finalDataList = handleDataList(dataList);
    const listLength = finalDataList.length;
    return react_1.useMemo(() => {
        return (react_1.default.createElement(react_native_1.ScrollView, { ref: $scroll, style: styles.content, removeClippedSubviews: false, scrollEventThrottle: 8, stickyHeaderIndices: [stickyIndex], onScroll: onScroll }, finalDataList.map((item, index) => {
            return (react_1.default.createElement(react_native_1.View, { key: index, onLayout: event => onItemLayout(event, index, index === listLength - 1) }, item.type && item.type === 'anchor' ? (react_1.default.createElement(__1.CellGroup, { key: index, title: item.name })) : (renderItem(item.data, index))));
        })));
    }, [dataList, stickyIndex]);
};
const styles = react_native_1.StyleSheet.create({
    content: {
        flex: 1,
    },
});
exports.default = List;
